# Copyright (c) 2011-2017 NAITOH Jun
# Released under the MIT license
# http://www.opensource.org/licenses/MIT

RBPDFFontDescriptor.define('err_font2') do |font|
  font[:type]='Type0'
  font[:cw]={}
end
