RBPDFFontDescriptor.define('courier') do |font|
  # core font definition file for RBPDF
  font[:type]='core'
  font[:dw]=600
  font[:cw] = {}
  0.upto(255) do |i|
  	font[:cw][i]=600
	end
end
RBPDFFontDescriptor.define('courierb') do |font|
  # core font definition file for RBPDF
  font[:type]='core'
  font[:dw]=600
  font[:cw] = {}
  0.upto(255) do |i|
  	font[:cw][i]=600
	end
end
RBPDFFontDescriptor.define('courierbi') do |font|
  # core font definition file for RBPDF
  font[:type]='core'
  font[:dw]=600
  font[:cw] = {}
  0.upto(255) do |i|
  	font[:cw][i]=600
	end
end
RBPDFFontDescriptor.define('courieri') do |font|
  # core font definition file for RBPDF
  font[:type]='core'
  font[:dw]=600
  font[:cw] = {}
  0.upto(255) do |i|
  	font[:cw][i]=600
	end
end

