# coding: UTF-8
#============================================================+
# Begin       : 2008-03-04
# Last Update : 2023-03-23
#
# Description : Example 009 for RBPDF class
#               Test Image (need rmagick)
#
# Author: Jun NAITOH
# License: LGPL 2.1 or later
#============================================================+

require("example_common.rb")

class Example009Controller < ApplicationController
  def index
    # create new PDF document
    pdf = RBPDF.new(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false)

    # set document information
    pdf.set_creator(PDF_CREATOR)
    pdf.set_author(PDF_AUTHOR)
    pdf.set_title('RBPDF Example 009')
    pdf.set_subject('RBPDF Tutorial')
    pdf.set_keywords('RBPDF, PDF, example, test, guide')

    # set default header data
    pdf.set_header_data(PDF_HEADER_LOGO, PDF_HEADER_LOGO_WIDTH, PDF_HEADER_TITLE + ' 009', PDF_HEADER_STRING)

    # set header and footer fonts
    pdf.set_header_font([PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN])
    pdf.set_footer_font([PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA])

    # set default monospaced font
    pdf.set_default_monospaced_font(PDF_FONT_MONOSPACED)

    # set margins
    pdf.set_margins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT)
    pdf.set_header_margin(PDF_MARGIN_HEADER)
    pdf.set_footer_margin(PDF_MARGIN_FOOTER)

    # set auto page breaks
    pdf.set_auto_page_break(true, PDF_MARGIN_BOTTOM)

    # set image scale factor
    pdf.set_image_scale(PDF_IMAGE_SCALE_RATIO)

    # set some language-dependent strings
    pdf.set_language_array($l)

    # ---------------------------------------------------------
    # add a page
    pdf.add_page()

    # set JPEG quality
    pdf.set_jpeg_quality(75)

    # image(file, x='', y='', w=0, h=0, type='', link='', align='', resize=false, dpi=300, palign='', ismask=false, imgmask=false, border=0, fitbox=false, hidden=false, fitonpage=false)

    # JPEG Image example (need rmagick)
    begin
      pdf.image(PDF_IMAGE_DEMO_JPG, 50, 50, 100, 150, 'JPG', 'https://github.com/naitoh/rbpdf', '', true, 150, '', false, false, 1, false, false, false)
    rescue => err
      logger.error "pdf: Image: error: #{err.message}"
    end
    # ---------------------------------------------------------
    # add a page
    pdf.add_page()

    # PNG Image example (need rmagick)
    begin
      pdf.image(PDF_IMAGE_DEMO_PNG, 50, 50, 100, 150, 'PNG', 'https://github.com/naitoh/rbpdf', '', true, 150, '', false, false, 1, false, false, false)
    rescue => err
      logger.error "pdf: Image: error: #{err.message}"
    end
    # ---------------------------------------------------------
    # add a page
    pdf.add_page()

    # WebP Image example (need rmagick)
    begin
      pdf.image(PDF_IMAGE_DEMO_WEBP, 50, 50, 100, 150, 'WEBP', 'https://github.com/naitoh/rbpdf', '', true, 150, '', false, false, 1, false, false, false)
    rescue => err
      logger.error "pdf: Image: error: #{err.message}"
    end

    # Close and output PDF document
    send_data pdf.output(), :type => "application/pdf", :disposition => "inline"
  end
end

#============================================================+
# END OF FILE                                                
#============================================================+
